#pragma OPENCL_EXTENSION cl_khr_fp_64 : enable
#define MAX_IT 45
#define MIN_DIST 0.005

typedef double2 Complex;

Complex multiply(Complex c1, Complex c2) {
    return (Complex) (c1.s0*c2.s0 - c1.s1*c2.s1,
	                  c1.s0*c2.s1 + c1.s1*c2.s0 );
}

Complex div(Complex c1, Complex c2) {
    double d = c2.s0*c2.s0 + c2.s1*c2.s1;
    
    return (Complex) ((c1.s0*c2.s0 + c1.s1*c2.s1)/d,
	                  (c1.s1*c2.s0 - c1.s0*c2.s1)/d );
}

__kernel void fraktal(write_only image2d_t output,
                      int width, int height,
                      double2 centerPt,
                      double scale,
                      double alpha) {
    size_t xPos = get_global_id(0);
    size_t yPos = get_global_id(1);
    double4 fcolor;
    uint4 color;
    Complex x;
	Complex xNeu;
	Complex f;
	Complex df;
	Complex quotient;
	double cr = ((double)((int)xPos - width/2) - centerPt.x)*scale;
	double ci = ((double)((int)yPos - height/2) - centerPt.y)*scale;
	x = (Complex)(cr, ci);
	double m;
	int i3, iMax;
	Complex sol1 = (Complex)(-1.0, 0.0);
    Complex sol2 = (Complex)(cos(radians(alpha)), sin(radians(alpha)));
    Complex sol3 = (Complex)(cos(radians(alpha)), -sin(radians(alpha)));
    Complex c1 = (Complex)(1.0 - 2.0*cos(radians(alpha)), 0);
    // c2 = vec2( 1.0 + 2.0*cos(radians(alpha)), 0);
    Complex c3 = (Complex)(1.0, 0.0);
    
	fcolor = (double4)(1.0, 1.0, 1.0, 1.0);
	for (int i = 0; i < MAX_IT; i++) {
		f = x + c1;
		df = x + f;
		f  = multiply(f, x) + c1;
		df = multiply (df, x) + f;
		f  = multiply(f, x) + c3;
		
		xNeu = x - div(f, df);
		x = xNeu;
        iMax = i;
		if (distance(x, sol1) < MIN_DIST) {
			break;
		} else if (distance(x, sol2) < MIN_DIST) {
			break;
		} else if (distance(x, sol3) < MIN_DIST) {
			break;
		}
	}
	i3 = (iMax + 1) / 3;
	iMax =  3*i3;
	m = (double)(MAX_IT + 4)/(double)(MAX_IT  + 4 - iMax);
	if (distance(x, sol1) < MIN_DIST) {
		fcolor = (double4) (0.65*m, 0.2*m, 0.2*m, 1.0);
	} else if (distance(x, sol2) < MIN_DIST) {
		fcolor = (double4) (0.2*m, 0.65*m, 0.2*m, 1.0);
	} else if (distance(x, sol3) < MIN_DIST) {
		fcolor = (double4) (0.2*m + 0.1, 0.2*m +0.1, 0.65*m, 1.0);
	} 
	fcolor = fcolor *255.0;
    color = (uint4) ((uint)fcolor.x,
                     (uint)fcolor.y,
                     (uint)fcolor.z,
                     (uint)fcolor.w);
    
    write_imageui(output, (int2)(xPos, height - yPos), color);

}
