#include "Window.h"
#include "OpenCLManager.h"
#include <stdbool.h>  //  für  true
#include <stdio.h>    //  für  printf

void setScale(GtkRange *widget, gpointer arg);
void setAlpha(GtkRange *widget, gpointer arg);

void destroy (GtkWidget *window, gpointer data)
{
	applicationModel *model = (applicationModel*) data;
	freeResources(&(model -> gpu));
    gtk_main_quit();
}

gboolean paint(GtkWidget* widget, cairo_t* context, gpointer arg)
{
	applicationModel *model = (applicationModel*) arg;
	windowElements *gui = &model -> gui;
	gpuModel *gpu = &model -> gpu;
    int h = gtk_widget_get_allocated_height(widget);
    int w = gtk_widget_get_allocated_width(widget);
	GdkPixbuf *pixbuf;
	int x0, y0;
	
	x0 = 0; //(int)(gui -> screenWidth - gui -> imageWidth)/2;
    y0 = 0; //(int)(gui -> screenWeight - gui -> imageWidth)/2; 
    cairo_save(context);
    
    pixbuf = gdk_pixbuf_new_from_data
                    ((*gpu).imageBuffer,
                     GDK_COLORSPACE_RGB, true, 8,
                     gpu -> imageSize,
                     gpu -> imageSize,
                     4*gpu -> imageSize, NULL, NULL); 
                      
    gdk_cairo_set_source_pixbuf(context, pixbuf, x0, y0);
    cairo_paint(context);
    cairo_restore(context);
	g_object_unref(pixbuf);
	return FALSE;
}

gboolean configureView(GtkWidget *widget, GdkEvent *event,
                       gpointer user_data) {
	applicationModel *model = (applicationModel*) user_data;
	gpuModel *gpu = &model -> gpu;
    int h = gtk_widget_get_allocated_height(widget);
    int w = gtk_widget_get_allocated_width(widget);
    int useableWidth  = (w/16) * 16;
    int useableHeight = (h/16) * 16;
    int s = useableWidth < useableHeight ? useableWidth : useableHeight;
    int size = s * s * 4;
    if (gpu -> imageBuffer != (char) 0) {
		freeImage(gpu);
    }
     
    gpu -> imageWidth = useableWidth;
    gpu -> imageHeight = useableHeight;
    gpu -> imageSize = s;
    
    printf("about to create image %d %d\n", h, w);
    createImage(gpu);
    printf("in configureView %d %d\n", h, w);
    performComputation(model);
	return TRUE;
}

GtkWidget* createWindow(applicationModel* model)
{
    GtkWidget* window;
    GtkWidget* vBox;
    GtkWidget *sliderZoom;
    GtkWidget *sliderAlpha;
    windowElements* windowStruct = &(model -> gui);
    gpuModel *gpu = &(model -> gpu);


    window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_title( GTK_WINDOW(window), "OpenCL Demo");
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_container_set_border_width(GTK_CONTAINER(window), 8);
    gtk_widget_set_size_request(window, 320, 240);
    
    vBox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
    windowStruct -> adjustmentZoom = gtk_adjustment_new(10.0, 3.0, 100.0, 1.0, 1.0, 1.0);
    windowStruct -> adjustmentAlpha = gtk_adjustment_new(60.0, 5.0, 176.0, 0.5, 0.5, 1.0);
    windowStruct -> paintArea = gtk_drawing_area_new();
    sliderZoom = gtk_scale_new(GTK_ORIENTATION_VERTICAL, windowStruct -> adjustmentZoom);
    sliderAlpha = gtk_scale_new(GTK_ORIENTATION_VERTICAL, windowStruct -> adjustmentAlpha);
    gtk_range_set_inverted(GTK_RANGE(sliderZoom), TRUE);
    gtk_range_set_inverted(GTK_RANGE(sliderAlpha), TRUE);
   
    g_signal_connect( G_OBJECT (window), "destroy",
                      G_CALLBACK(destroy), (gpointer) model);
    g_signal_connect( sliderZoom, "value_changed",
                      G_CALLBACK(setScale), (gpointer) model);
    g_signal_connect( sliderAlpha, "value_changed",
                      G_CALLBACK(setAlpha), (gpointer) model);
    g_signal_connect( windowStruct -> paintArea, "draw",
                     G_CALLBACK(paint), (gpointer) model);
    g_signal_connect( windowStruct -> paintArea, "configure-event",
                     G_CALLBACK(configureView), (gpointer) model);
                     
    gtk_container_add(GTK_CONTAINER(window), vBox);
    gtk_box_pack_start(GTK_BOX(vBox), sliderZoom, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vBox), windowStruct -> paintArea, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vBox), sliderAlpha, FALSE, FALSE, 0);

    gtk_widget_show(sliderZoom);
    gtk_widget_show(windowStruct -> paintArea);
    gtk_widget_show(sliderAlpha);
    gtk_widget_show(vBox);            
    gtk_widget_show(window);
    return window;
}

void setScale(GtkRange *widget, gpointer arg) {
	applicationModel *model = (applicationModel*) arg;
	windowElements *gui = &model -> gui;
	gpuModel *gpu = &model -> gpu;
	performComputation(model);
	gtk_widget_queue_draw(gui -> paintArea);
}

void setAlpha(GtkRange *widget, gpointer arg) {
	applicationModel *model = (applicationModel*) arg;
	windowElements *gui = &model -> gui;
	gpuModel *gpu = &model -> gpu;
	performComputation(model);
	gtk_widget_queue_draw(gui -> paintArea);
}
